@@ -28,9 +28,14 @@
 #include <gdk/gdkkeysyms.h>
 #include <gconf/gconf-client.h>
 
+#ifdef HAVE_CANBERRA_GTK
+#include <canberra-gtk.h>
+#endif
+
 #include "drwright.h"
 #include "drw-utils.h"
 #include "drw-break-window.h"
+#include "drw-timer.h"
 
 struct _DrwBreakWindowPrivate {
 	GtkWidget *clock_label;
@@ -40,7 +45,7 @@ struct _DrwBreakWindowPrivate {
 	GtkWidget *postpone_entry;
 	GtkWidget *postpone_button;
 
-	GTimer    *timer;
+	DrwTimer  *timer;
 
 	gint       break_time;
 	gchar     *break_text;
@@ -195,9 +200,9 @@ drw_break_window_init (DrwBreakWindow *window)
 			g_source_remove (priv->postpone_sensitize_id);
 		}
 
-		priv->postpone_sensitize_id = g_timeout_add (500,
-							     (GSourceFunc) postpone_sensitize_cb,
-							     window);
+		priv->postpone_sensitize_id = g_timeout_add_seconds (5,
+								     (GSourceFunc) postpone_sensitize_cb,
+								     window);
 
 		g_signal_connect (priv->postpone_button,
 				  "clicked",
@@ -266,7 +271,7 @@ drw_break_window_init (DrwBreakWindow *window)
 
 	gtk_window_stick (GTK_WINDOW (window));
 
-	priv->timer = g_timer_new ();
+	priv->timer = drw_timer_new ();
 
 	/* Make sure we have a valid time label from the start. */
 	clock_timeout_cb (window);
@@ -274,6 +279,9 @@ drw_break_window_init (DrwBreakWindow *window)
 	priv->clock_timeout_id = g_timeout_add (1000,
 						(GSourceFunc) clock_timeout_cb,
 						window);
+#ifdef HAVE_CANBERRA_GTK
+	ca_context_play (ca_gtk_context_get (), 0, CA_PROP_EVENT_ID, "desktop-screen-lock", NULL);
+#endif
 }
 
 static void
@@ -312,7 +320,7 @@ drw_break_window_dispose (GObject *object)
 	priv = window->priv;
 
 	if (priv->timer) {
-		g_timer_destroy (priv->timer);
+		drw_timer_destroy (priv->timer);
 		priv->timer = NULL;
 	}
 
@@ -375,7 +383,7 @@ clock_timeout_cb (DrwBreakWindow *window)
 
 	priv = window->priv;
 
-	seconds = 1 + priv->break_time - g_timer_elapsed (priv->timer, NULL);
+	seconds = 1 + priv->break_time - drw_timer_elapsed (priv->timer);
 	seconds = MAX (0, seconds);
 
 	if (seconds == 0) {
@@ -385,6 +393,9 @@ clock_timeout_cb (DrwBreakWindow *window)
 		 */
 		priv->clock_timeout_id = 0;
 
+#ifdef HAVE_CANBERRA_GTK
+		ca_context_play (ca_gtk_context_get (), 0, CA_PROP_EVENT_ID, "alarm-clock-elapsed", NULL);
+#endif
 		g_signal_emit (window, signals[DONE], 0, NULL);
 
 		return FALSE;
